<?php
/**
 * @file
 * List handler for CCK reference field backed lists. Includes entity handlers
 * for CCK node and user reference fields on which the cck list handler is dependent.
 */

$plugin = array(
  'id' => 'noderef',
  'content type' => 'node',
  'label' => t('Nodes (by cck node reference field)'),
  'handler' => array(
    'class' => 'lister_cckrefs_noderef_entity_handler',
    'parent' => 'lister_node_entity_handler',
  ),
  'associated list handler' => 'lister_refs',
);

/**
 * Implements lister_entity_handler for nodes that will be referenced by a cck node reference field.
 * Should be constructed with 'field_info' in the settings array.
 */
class lister_cckrefs_noderef_entity_handler extends node_lister_entity_handler {
  function get_settings_form($settings) {
    $form = array();

    if (!empty($this->settings['field_info'])) {
      $url = url('admin/content/node-type/'. $this->settings['field_info']['type_name'] .'/fields/'. $this->settings['field_info']['field_name'], array('query' => drupal_get_destination()));
      $referenceable_types = array_filter($this->settings['field_info']['referenceable_types']);

      $description = t('These options are set by the CCK node reference field settings.');
      $description .= t('Field settings can changed on the content type\'s <a href="@field_page_url">field management page</a>. ', array('@field_page_url' => $url));


      $form['types'] = array(
        '#type' => 'checkboxes',
        '#disabled' => TRUE,
        '#options' => array_map('check_plain', node_get_types('names')),
        '#title' => t('Content (Node) types'),
        '#description' => $description,
        '#default_value' => $referenceable_types,
      );
    }
    else {
      $form['message'] = array(
        '#value' => t('Supported entities are determined by the selected reference field.'),
      );
    }

    return $form;
  }

  /**
   * Override is_entity_supported to delegate to the CCK field settings. Don't
   * fully delegate as that would entail running a query and or a view to get the
   * set of allowed nodes.
   */
  function is_entity_supported($type, $object) {
    $referenceable_types = array_filter($this->settings['field_info']['referenceable_types']);
    return $type == 'node' && in_array($object->type, $referenceable_types);
  }
}


