<?php
/**
 * @file
 * List handler for CCK reference field backed lists. Includes entity handlers
 * for CCK node and user reference fields on which the cck list handler is dependent.
 */

$plugin = array(
  'id' => 'userref',
  'content type' => 'user',
  'label' => t('Users (by cck user reference field)'),
  'handler' => array(
    'class' => 'lister_cckrefs_userref_entity_handler',
    'parent' => 'lister_user_entity_handler', // name of plugin parent - when classloading the 'handler' class of parent will be loaded first
  ),
  'associated list handler' => 'lister_refs',
);

/**
 * Implements lister_entity_handler for nodes that will be referenced by a cck node reference field.
 * Should be constructed with 'field_info' in the settings array.
 */
class lister_cckrefs_userref_entity_handler extends user_lister_entity_handler {
  function get_settings_form($settings) {
    $form = array();

    if (!empty($this->settings['field_info'])) {
      $url = url('admin/content/node-type/'. $this->settings['field_info']['type_name'] .'/fields/'. $this->settings['field_info']['field_name'], array('query' => drupal_get_destination()));

      if (module_exists('userreference')) {
        $form = userreference_field_settings('form', $this->settings['field_info']);
        unset($sub_form['advanced']);
        $form['referenceable_roles']['#disabled'] = TRUE;
        $form['referenceable_status']['#disabled'] = TRUE;
      }

      $description = t('These options are set by the CCK node reference field settings.');
      $description .= ' '. t('Field settings can changed on the content type\'s <a href="@field_page_url">field management page</a>. ', array('@field_page_url' => $url));

      $form['#description'] = $description;
    }
    else {
      $form['message'] = array(
        '#value' => t('Supported entities are determined by the selected reference field.'),
      );
    }

    return $form;
  }

  /**
   * Override is_entity_supported to delegate to the CCK field settings. Don't
   * fully delegate as that would entail running a query and or a view to get the
   * set of allowed nodes.
   */
  function is_entity_supported($type, $object) {
    if ($type == 'user') {
      if ($field = $this->settings['field_info']) {
        if (isset($field['referenceable_roles']) && is_array($field['referenceable_roles'])) {
          // keep only selected checkboxes
          $roles = array_filter($field['referenceable_roles']);
          // filter invalid values that seems to get through sometimes ??
          $roles = array_intersect(array_keys(user_roles(1)), $roles);
          if (!empty($roles) && !in_array(DRUPAL_AUTHENTICATED_RID, $roles)) {
            // if not one of these roles then return FALSE
            $matching_roles = array_intersect(array_keys($roles), array_keys($object->roles));
            if (empty($matching_roles)) {
              return FALSE;
            }
          }
        }

        if (isset($field['referenceable_status']) && is_numeric($field['referenceable_status'])) {
          // if not one of these statuses then return FALSE
          if ($field['referenceable_status'] <> $object->status) {
            return FALSE;
          }
        }
      }
      return TRUE;
    }
  }
}



